<?php
namespace Teknisa\PLA\Modules\Parametros\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response\Error;
use Teknisa\GER\Services\CrudService;
use Doctrine\DBAL\Connection;
use Exception;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Pla06400MetaParaRestoIngestaSobraLimpa extends \Teknisa\GER\Controllers\TeknisaCrud {

	protected $connection;
	protected $util;
	protected $environment;
	protected $crudService;
	protected $parameterBag;
	protected $pla06400MetaParaRestoIngestaSobraLimpaService;
	protected $dataSourceName = "metarisl";
	
    public function __construct($crudService, $parameterBag, $connection, $util, $environment, $pla06400MetaParaRestoIngestaSobraLimpaService) {
        $this->crudService   = $crudService;
        $this->parameterBag  = $parameterBag;
        $this->connection    = $connection;
        $this->util          = $util;
        $this->environment   = $environment;
        $this->pla06400MetaParaRestoIngestaSobraLimpaService = $pla06400MetaParaRestoIngestaSobraLimpaService;
    }
    
    
    public function save(DTO\Request $request, DTO\Response $response){
        $dados = $request->getDataSet()->getRows();
        $this->saveDadosFiliais($dados);
    }
    
    public function deleteDate(DTO\Request $request, DTO\Response $response){
        $rows = $request->getDataSet()->getRows();
        foreach ($rows as $row) {
            $this->pla06400MetaParaRestoIngestaSobraLimpaService->deleteDate($row);
        }
    }
    
    public function exportMetas(DTO\Request $request, DTO\Response $response){
        $dados = $request->getDataSet()->getRows();
        $cdfilial = $this->getCdFilial($dados);
        foreach ($cdfilial as $codFil) {
            $dados[0]['CDFILIAL'] = $codFil;
            $this->saveDadosFiliais($dados);
        }
    }
    
    public function saveDadosFiliais($dados){
        foreach ($dados as $dado) {
            
            $conditions = array(
                array(
                    "columnName" => "CDFILIAL", 
                    "operator" => "=",
                    "value" => $dado["CDFILIAL"]
                ),
                array(
                    "columnName" => "DTREFMETRISL", 
                    "operator" => "=",
                    "value" => $dado['MES']
                )
            );
            
            $dataSet = $this->crudService->getFilteredDataSet($this->dataSourceName, $conditions)->getRows();
            
            if(empty($dataSet)) {
                
                $row = array(
                    "DTREFMETRISL" => $dado['MES'],
                    "VRPEMETAREIN" => str_replace(",", ".", $dado['VRPEMETAREIN']),
                    "VRPEMETASOLI" => str_replace(",", ".", $dado['VRPEMETASOLI']),
                    "__is_new"     => true,
                    "CDFILIAL"     => $dado['CDFILIAL']    
                );
                
                $row = new DataSet($this->dataSourceName, array($row));
                $newRequest = new DTO\Request\DataSet($row, '', '', '');
                $rowsToSave = $this->crudService->getRowsToSave($newRequest,$this->dataSourceName);
                
                $this->crudService->save($this->dataSourceName, $rowsToSave);
                
            } else {
                
                $dataSet[0]['VRPEMETAREIN'] = str_replace(",", ".", $dado['VRPEMETAREIN']);
                $dataSet[0]['VRPEMETASOLI'] = str_replace(",", ".", $dado['VRPEMETASOLI']);
                
                $dataSet = new DataSet($this->dataSourceName, $dataSet);
                $newRequest = new DTO\Request\DataSet($dataSet, '', '', '');
                $rowsToSave = $this->crudService->getRowsToSave($newRequest,$this->dataSourceName);
                
                $this->crudService->save($this->dataSourceName, $rowsToSave);
            }
        }
    }
    
    public function getCdFilial($dados){
        $cdfilial = array();
        
        if($dados[0]['ALLCDFILIAL'] == 'S')
        {
            $dataSourceNameAnterior = $this->dataSourceName;
            $this->dataSourceName = 'filial_filioper';
            $conditions = array(
                array(
                    "columnName" => "CDOPERADOR", 
                    "operator" => "=",
                    "value" => $dados[0]['CDOPERADOR']
                )
            );
            
            $allfilial = $this->crudService->getFilteredDataSet($this->dataSourceName, $conditions)->getRows();
            foreach ($allfilial as $filial) {
                array_push($cdfilial,$filial['CDFILIAL']);
            }
            
            $this->dataSourceName = $dataSourceNameAnterior;
        }else
        {
            if(is_array($dados[0]['CDFILIAL']))
            {
                foreach($dados[0]['CDFILIAL'] as $dado)
                {
                    array_push($cdfilial, $dado);    
                }
            }else
            {
                array_push($cdfilial, $dados[0]['CDFILIAL']);
            }
        }
        
        return $cdfilial;
    }
    
    public function saveTabbedChanges(DTO\Request $request, DTO\Response $response){
        $dataset = $request->getParameters()['dataset'];
        
        try{
            if($dataset['ORIGIN']['value']=="R"){
                if($dataset['ROW']['changed']){
                    $this->pla06400MetaParaRestoIngestaSobraLimpaService->saveRestoIngestaTabChanges($dataset['ROW']);
                }
                
                $arrayDataToInclude = array_filter($dataset['DATA'], function($element){
                    return $element['IDCUSTOREINSOLI']=="R";
                });
                $arrayDataToDelete = array_filter($dataset['DATA'], function($element){
                    return $element['IDCUSTOREINSOLI']=="X";
                });
                
                
                foreach($arrayDataToInclude as $key => $element){
                    $this->pla06400MetaParaRestoIngestaSobraLimpaService->includeGrupoTipoPrato($element, $dataset['ROW']['CDFILIAL']);
                } 
                foreach($arrayDataToDelete as $key => $element){
                    $element['IDCUSTOREINSOLI'] = 'R';
                    $this->pla06400MetaParaRestoIngestaSobraLimpaService->deleteGrupoTipoPrato($element, $dataset['ROW']['CDFILIAL']);
                }
                
            } else if($dataset['ORIGIN']['value']=="S"){
                
                if($dataset['ROW']['changed']){
                    $this->pla06400MetaParaRestoIngestaSobraLimpaService->saveSobraLimpaTabChanges($dataset['ROW']);
                }
                
                $arrayDataToInclude = array_filter($dataset['DATA'], function($element){
                    return $element['IDCUSTOREINSOLI']=="S";
                });
                $arrayDataToDelete = array_filter($dataset['DATA'], function($element){
                    return $element['IDCUSTOREINSOLI']=="X";
                });
                
                foreach($arrayDataToInclude as $key => $element){
                    $this->pla06400MetaParaRestoIngestaSobraLimpaService->includeGrupoTipoPrato($element, $dataset['ROW']['CDFILIAL']);
                }   
                foreach($arrayDataToDelete as $key => $element){
                    $element['IDCUSTOREINSOLI'] = 'S';
                    $this->pla06400MetaParaRestoIngestaSobraLimpaService->deleteGrupoTipoPrato($element, $dataset['ROW']['CDFILIAL']);
                }
            }
        } catch (\Exception $e){
            $response->setError(new Error("Error on save: ", $e->getMessage(), 500));
        }
    }
}