<?php
namespace Teknisa\PLA\Modules\Relatorio\Controllers;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Teknisa\GER\Utilities\Environment;

class Pla11400MapaDeProducao extends \Zeedhi\Framework\Controller\Report {

	protected $crudService;
	protected $novoCodigo;
	protected $environment;
	protected $procedureNutriente;
	protected $parameterBagColumns;
	protected $dataSourceManager;
	protected $entityManager;
	protected $mapaDeProducaoService;
	protected $qr2Strategy;
	protected $plaUtil;
	protected $birtStrategy;

	public function __construct($reportService, $mapaDeProducaoService, $crudService, $novoCodigo, $environment, $dataSourceManager, $entityManager, $qr2Strategy, $plaUtil, $birtStrategy) {
		$this->mapaDeProducaoService = $mapaDeProducaoService;
		$this->reportService = $reportService;
		$this->crudService = $crudService;
		$this->novoCodigo = $novoCodigo;
		$this->qr2Strategy = $qr2Strategy;
        $this->environment = $environment;
		$this->dataSourceManager = $dataSourceManager;
		$this->entityManager = $entityManager;
		$this->plaUtil = $plaUtil;
		$this->birtStrategy = $birtStrategy;
	}

	protected $reportMapping = array (
		'PLA11400' =>
		array (
			'strategy' => 'BIRT',
			'parameterMapping' =>
			array (
				"IDQUEBRAPG" => "P_IDQUEBRAPAGINA",
			  	"NRCONSPLAN" => "P_NRCONSPLAN",
			  	"NRORG"      => "P_NRORG"
			)
		)
	);
	public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
		$row = $request->getRow();
		$this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
		$row['DTINICIAL'] = $row['DTCARDAPIOINI'];
		$row['DTFINAL'] = $row['DTCARDAPIOFIN'];
		$this->getFiliais($request, $response);
		$filialServico = $this->getFilialServico($request, $response);
		$nrLista = $this->plaUtil->insertLista($filialServico);
		$nrConsPlan = $this->executaConsPlanGeral($row, $nrLista);
		$row['NRCONSPLAN']= $nrConsPlan['P_NRCONSPLAN'];

	}
	protected function getFiliais($request, $response){
		$row = $request->getRow();
		if($row['NMFILIAL']=="T"){
			if(!empty($row['CDGRUPFILI'])){
				$conditions= array(
					array(
						'columnName' => 'CDGRUPFILI',
						'operator' => '=',
						'value' => $row['CDGRUPFILI']
					),
					array(
						"columnName" =>'NRORG',
						"operator" => "=",
						"value" => $this->environment->getNrOrgTrab()
					),
					array(
						"columnName" =>'IDCNTRRESC',
						"operator" => "=",
						"value" => 'N'
					)
				);
				
				
				$this->parameterBagColumns = array("CDGRUPFILI", "NRORG");
				$filterCriteria = new FilterCriteria('filial_filgrupfili', $conditions);
				$newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
				$dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
			} else{
				$grupfili = isset($row['CDGRUPFILI']) ? $row['CDGRUPFILI'] : 'T';
				$conditions = array(
		            array(
		                'columnName' => 'CDGRUPFILI',
		                'operator' => '=',
		                'value' => $grupfili
		            ),
		            array(
		                'columnName' => 'CDOPERADOR',
		                'operator' => '=',
		                'value' => $this->environment->getCdOperador()
		            ),
		            array(
		                'columnName' => 'NRORG',
		                'operator' => '=',
		                'value' => $this->environment->getNrOrg()
		            ),
		            array(
		               'columnName' => 'IDCNTRRESC',
		                'operator' => '=',
		                'value' => 'N'
		            )
		        );
		        
		        $filterCriteria = new FilterCriteria('filial_operador_grupfili', $conditions);
		        $paramBag = array("NRORG", "CDOPERADOR", "CDGRUPFILI");
		        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $paramBag);
		        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
			}
			$row['NMFILIAL']=array();
			foreach ($dataSet->getRows() as $filial) {
				array_push($row['NMFILIAL'], $filial['CDFILIAL']);
			}
		}
	}
	protected function getFilialServico($request, $response){
		$row = $request->getRow();
		$filialServico=array();
		
		if($row['NMSERVICO']=="T") {
			foreach($row['NMFILIAL'] as $filial) {
				$conditions = array(
					array(
						'columnName' => 'CDFILIAL',
						'operator' => '=',
						'value' => $filial
					),
					array(
						'columnName' => 'CDGRUPFILI',
						'operator' => '=',
						'value' => !empty($row['CDGRUPFILI']) ? $row['CDGRUPFILI'] : "T"
					),
					array(
						"columnName" =>'NRORG',
						"operator" => "=",
						"value" => $this->environment->getNrOrgTrab()
					),
					array(
						'columnName' => 'CDGRUPSERV',
						'operator' => '=',
						'value' => ($row['NMGRUPSERV'] == "T") ? $row['NMGRUPSERV'] : join("_" , $row['NMGRUPSERV'])
					)
				);
				$this->parameterBagColumns = array("CDFILIAL", "CDGRUPSERV", "CDGRUPFILI", "NRORG");
				$filterCriteria = new FilterCriteria('servico_filial_grupserv_filgrupfili', $conditions);
				$newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
				$dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
				foreach ($dataSet->getRows() as $servico) {
					array_push($filialServico, $filial.$servico['CDSERVFILI']);
				}
			}
		} else {
			$filialServico = $row['NMSERVICO'];
		}
		return $filialServico;
	}
	
	protected function insereLista($filialServico, $nrLista) {
		$rowsToSave =array();
		foreach($filialServico as $filiServ){
			array_push($rowsToSave, array("NRLISTA"=>$nrLista,"CDLISTA"=>$filiServ,"__is_new"=>true));
		}
		$newDataSet = new DataSet(null , $rowsToSave);
		$newRequest = new DTO\Request\DataSet($newDataSet, "", "", "");
		$rows = $this->crudService->getRowsToSave($newRequest,'lista');
		$this->crudService->save('lista', $rows);
	}
	
	protected function executaConsPlanGeral($row, $nrLista) {
		$consplanGeral = new StoredProcedure($this->entityManager->getConnection(), 'CONSPLAN_GERAL');
		$consplanGeral->addParam(new Param('P_NRLISTA',      Param::PARAM_INPUT, $nrLista,            					Param::PARAM_TYPE_STR));
		$consplanGeral->addParam(new Param('P_DTINICIAL',    Param::PARAM_INPUT, $row['DTINICIAL'], 					Param::PARAM_TYPE_STR));
		$consplanGeral->addParam(new Param('P_DTFINAL',      Param::PARAM_INPUT, $row['DTFINAL'],   					Param::PARAM_TYPE_STR));
		$consplanGeral->addParam(new Param('P_IDPRODUTOS',   Param::PARAM_INPUT, 'CXX',               					Param::PARAM_TYPE_STR));
		$consplanGeral->addParam(new Param('P_IDSEPPRATO',   Param::PARAM_INPUT, 'P',                 					Param::PARAM_TYPE_STR));
		$consplanGeral->addParam(new Param('P_PEACRESCIMO',  Param::PARAM_INPUT, 0,                   					Param::PARAM_TYPE_INT));
		$consplanGeral->addParam(new Param('P_IDARREDOND',   Param::PARAM_INPUT, 'N',                 					Param::PARAM_TYPE_STR));
		$consplanGeral->addParam(new Param('P_CDPRODUTOINI', Param::PARAM_INPUT, '0',                 					Param::PARAM_TYPE_STR));
		$consplanGeral->addParam(new Param('P_CDPRODUTOFIN', Param::PARAM_INPUT, 'zzzzzzzzzz',        					Param::PARAM_TYPE_STR));
		$consplanGeral->addParam(new Param('P_CDOPERADOR',   Param::PARAM_INPUT, $this->environment->getCdOperador(),  	Param::PARAM_TYPE_STR));
		$consplanGeral->addParam(new Param('P_NRCONSPLANTH', Param::PARAM_INPUT, 'GERANOVCOD',       					Param::PARAM_TYPE_STR));
		$consplanGeral->addParam(new Param('P_NRCONSPLAN',   Param::PARAM_OUTPUT, null,              					Param::PARAM_TYPE_STR, 20));
		return $consplanGeral->execute();
	}

}